<?php

declare(strict_types = 1);

namespace Drupal\config_revision\Form;

use Drupal\Component\Diff\Diff;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Diff\DiffFormatter;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Form\RevisionRevertForm as CoreRevisionRevertForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Serialization\Yaml;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Revision revert form class.
 */
class RevisionRevertForm extends CoreRevisionRevertForm {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The diff formatter.
   *
   * @var \Drupal\Core\Diff\DiffFormatter
   */
  protected $diffFormatter;

  /**
   * Creates RevisionRevertForm object.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $bundle_information
   *   The bundle information.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The bundle information.
   * @param \Drupal\Core\Diff\DiffFormatter $diff_formatter
   *   The diff formatter.
   */
  public function __construct(DateFormatterInterface $date_formatter, EntityTypeBundleInfoInterface $bundle_information, EntityTypeManagerInterface $entity_type_manager, DiffFormatter $diff_formatter) {
    parent::__construct($date_formatter, $bundle_information);
    $this->entityTypeManager = $entity_type_manager;
    $this->diffFormatter = $diff_formatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_type.manager'),
      $container->get('diff.formatter')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $_entity_revision = NULL, Request $request = NULL) {
    $form = parent::buildForm($form, $form_state, $_entity_revision, $request);
    $current = $this->entityTypeManager
      ->getStorage($this->revision->getEntityTypeId())
      ->load($this->revision->id());
    $from = explode("\n", Yaml::encode($current->getConfigMap()->toArray()));
    $to = explode("\n", Yaml::encode($this->revision->getConfigMap()->toArray()));
    $diff = new Diff($from, $to);
    $this->diffFormatter->show_header = FALSE;
    // Add the CSS for the inline diff.
    $form['#attached']['library'][] = 'system/diff';

    $form['diff'] = [
      '#type' => 'table',
      '#attributes' => [
        'class' => ['diff'],
      ],
      '#header' => [
        ['data' => $this->t('From'), 'colspan' => '2'],
        ['data' => $this->t('To'), 'colspan' => '2'],
      ],
      '#rows' => $this->diffFormatter->format($diff),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\Core\Config\Entity\ConfigEntityStorage::doSave
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\config_revision\Entity\ConfigRevisionTypeInterface $config_revision_type */
    $config_revision_type = $this->revision->type->entity;
    /** @var \Drupal\Core\Config\Entity\ConfigEntityTypeInterface $definition */
    $definition = $this->entityTypeManager
      ->getDefinition($config_revision_type->id());
    $prefix = $definition->getConfigPrefix() . '.';
    $id = $this->revision->label();
    $config_name = $prefix . $id;
    $config = $this->configFactory()->getEditable($config_name);
    $config->setData($this->revision->getConfigMap()->toArray());
    $config->save(TRUE);
    $this->entityTypeManager
      ->getStorage($config_revision_type->id())
      ->resetCache([$id]);
    parent::submitForm($form, $form_state);
  }

}
