<?php

declare(strict_types = 1);

namespace Drupal\Tests\config_revision\Functional;

use Drupal\config_revision\Entity\ConfigRevision;
use Drupal\config_revision\Entity\ConfigRevisionType;
use Drupal\Core\Url;
use Drupal\Tests\BrowserTestBase;
use Symfony\Component\HttpFoundation\Response;

/**
 * Tests config revision entity workflow.
 *
 * @group config_revision
 */
class ConfigRevisionWebformTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'config_revision',
    'webform',
    'block',
  ];

  /**
   * The admin user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $adminUser;

  /**
   * The another user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $anotherUser;

  /**
   * Permissions to grant admin user.
   *
   * @var array
   */
  protected $permissions = [
    'administer config_revision',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'classy';

  /**
   * Sets the test up.
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser($this->permissions);
    $this->anotherUser = $this->drupalCreateUser();
    // Enable webform revision.
    $config = $this->config('config_revision.settings');
    $config->set('enabled_entity_types', ['webform']);
    $config->save();
    $webform_config_revision_type = ConfigRevisionType::create([
      'id' => 'webform',
      'label' => 'Webform',
      'description' => '',
    ]);
    $webform_config_revision_type->save();
  }

  /**
   * Tests the config revision for webform.
   */
  public function testConfigRevisionWebform() {
    $this->drupalLogin($this->adminUser);
    $this->drupalPlaceBlock('page_title_block');
    $this->drupalPlaceBlock('local_actions_block');
    $this->drupalPlaceBlock('local_tasks_block');
    $assert_session = $this->assertSession();
    // Create webform.
    $webform_storage = \Drupal::entityTypeManager()->getStorage('webform');
    $elements = [
      'root' => [
        '#type' => 'textfield',
        '#title' => 'root',
      ],
      'container' => [
        '#type' => 'container',
        '#title' => 'container',
        'child' => [
          '#type' => 'textfield',
          '#title' => 'child',
          '#default_value' => '{default value}',
        ],
      ],
    ];
    /** @var \Drupal\webform\WebformInterface $webform */
    $webform = $webform_storage->create(['id' => 'webform_test']);
    $webform->setElements($elements);
    $webform->setStatus(TRUE);
    $webform->save();
    $webform = $webform_storage->loadUnchanged('webform_test');
    $this->drupalGet($webform->toUrl());
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $assert_session->fieldExists('root');
    $assert_session->fieldExists('child');
    $webform_revision = ConfigRevision::loadConfigRevisionByConfigId('webform_test');
    $this->drupalGet('/admin/content/config-revision');
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $assert_session->linkByHrefExists($webform_revision->toUrl()->toString());
    $this->drupalGet($webform_revision->toUrl());
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $this->drupalGet($webform_revision->toUrl('edit-form'));
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $assert_session->fieldExists('revision_log[0][value]');
    $this->drupalGet($webform_revision->toUrl('delete-form'));
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $assert_session->pageTextContainsOnce('This action cannot be undone.');

    // Save a webform revision.
    $webform->deleteElement('container');
    $webform->save();
    $this->drupalGet($webform->toUrl());
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $this->assertField('root');
    $this->assertNoField('child');
    $this->drupalGet($webform_revision->toUrl('version-history'));

    // Revert back to the first revision.
    $this->drupalGet(Url::fromRoute('entity.config_revision.revision_revert_form', [
      'config_revision' => 1,
      'config_revision_revision' => 1,
    ]));
    $this->submitForm([], 'Revert');
    $this->drupalGet($webform->toUrl());
    $assert_session->statusCodeEquals(Response::HTTP_OK);
    $this->assertField('root');
    $this->assertField('child');
    $webform = $webform_storage->loadUnchanged('webform_test');
    // Check elements decoded and flattened.
    $flattened_elements = [
      'root' => [
        '#type' => 'textfield',
        '#title' => 'root',
      ],
      'container' => [
        '#type' => 'container',
        '#title' => 'container',
      ],
      'child' => [
        '#type' => 'textfield',
        '#title' => 'child',
        '#default_value' => '{default value}',
      ],
    ];
    $this->assertEquals($webform->getElementsDecodedAndFlattened(), $flattened_elements);
  }

}
